from bs4 import BeautifulSoup
import sys
import os
import json
import urllib.request, urllib.error, urllib.parse


# code by
# stephenhouser/Bing-Image-Scraper
# https://gist.github.com/stephenhouser/c5e2b921c3770ed47eb3b75efbc94799


def _get_soup(url, header):
    # return BeautifulSoup(urllib2.urlopen(urllib2.Request(url,headers=header)),
    # 'html.parser')
    return BeautifulSoup(urllib.request.urlopen(
        urllib.request.Request(url, headers=header)),
        'html.parser')


if __name__ == '__main__':
    query = sys.argv[1]
    query = query.split()
    query = '+'.join(query)
    query += '+filterui:imagesize-large'
    url = "http://www.bing.com/images/search?q=" + query + "&FORM=HDRSC2&adlt=off"

    # add the directory for your image here
    DIR = "Pictures"
    header = {
        'User-Agent': "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.134 Safari/537.36"}
    soup = _get_soup(url, header)

    ActualImages = []  # contains the link for Large original images, type of  image
    for a in soup.find_all("a", {"class": "iusc"}):
        # print a
        mad = json.loads(a["mad"])
        turl = mad["turl"]
        m = json.loads(a["m"])
        murl = m["murl"]

        image_name = urllib.parse.urlsplit(murl).path.split("/")[-1]
        print(image_name)

        ActualImages.append((image_name, turl, murl))

    print("there are total", len(ActualImages), "images")

    if not os.path.exists(DIR):
        os.mkdir(DIR)

    DIR = os.path.join(DIR, query.split()[0])
    if not os.path.exists(DIR):
        os.mkdir(DIR)

    ##print images
    for i, (image_name, turl, murl) in enumerate(ActualImages):
        try:
            # req = urllib2.Request(turl, headers={'User-Agent' : header})
            # raw_img = urllib2.urlopen(req).read()
            # req = urllib.request.Request(turl, headers={'User-Agent' : header})
            raw_img = urllib.request.urlopen(murl).read()

            cntr = len([i for i in os.listdir(DIR) if image_name in i]) + 1
            # print cntr

            f = open(os.path.join(DIR, image_name), 'wb')
            f.write(raw_img)
            f.close()
            print('downloaded ' + murl)
        except Exception as e:
            print("could not load : " + image_name)
            print(e)



from postcards.postcards import Postcards
from postcards.plugin_pexels.util.pexels import get_random_image_url, read_from_url
import sys
import json
import os
import random
import nltk

jokes_location = os.path.dirname(os.path.realpath(__file__)) + '/chuck_norris_jokes.json'
nltk.download('averaged_perceptron_tagger')
nltk.download('punkt')


class PostcardsChuckNorris(Postcards):
    """
    Send a postcard with a chuck norris joke.
    Parse the joke for nouns and choose a matching picture on pexels.com

    """

    def enhance_send_subparser(self, parser):
        parser.add_argument('--category', default=None, type=str,
                            help='choose a custom joke category: nerdy, explicit')
        parser.add_argument('--duplicate-file', default=None, type=str,
                            help='avoid sending the same joke twice by setting a file which stores already sent jokes')

    def get_img_and_text(self, plugin_payload, cli_args):
        jokes = self._read_jokes()

        if cli_args.category:
            jokes = self._filter_by_category(jokes, cli_args.category)

        if not jokes:
            self.logger.error('No jokes found for category: {}'.format(cli_args.category))
            exit(1)

        exclude_jokes = []

        if cli_args.duplicate_file:
            exclude_file = self._make_absolute_path(cli_args.duplicate_file)
            if os.path.isfile(exclude_file):
                with open(exclude_file) as f:
                    content = f.readlines()
                    exclude_jokes = [x.strip() for x in content]

        jokes = self._filter_by_exclude_id(jokes, exclude_jokes)

        if not jokes:
            self.logger.error('No more jokes to choose from. everything excluded by {}'.format(cli_args.duplicate_file))
            exit(1)

        joke = random.choice(jokes)
        postcard_text = joke.get('joke')
        nouns = self._find_nouns(postcard_text)

        keyword = ''
        counter = 0
        for n in nouns:
            if counter > 2:
                break
            keyword += n + ' '
            counter = counter + 1
            self.logger.debug(n)

        keyword = keyword.strip()
        if keyword:
            try:
                url = get_random_image_url(keyword=keyword)
            except Exception:
                url = get_random_image_url()
        else:
            url = get_random_image_url()

        self.logger.debug('keyword: {}, text: {}'.format(keyword, postcard_text))
        self.logger.debug('url: {}'.format(url))

        if cli_args.duplicate_file:
            with open(exclude_file, "a") as excludes:
                excludes.write(str(joke.get('id')) + '\n')

        return {
            'img': read_from_url(url),
            'text': postcard_text
        }

    def _read_jokes(self):
        with open(jokes_location, 'r') as f:
            jokes = json.loads(f.read())
            return jokes.get('value')

    def _find_nouns(self, sentence):
        tokens = nltk.word_tokenize(sentence)
        tagged = nltk.pos_tag(tokens)
        nouns = [word for word, pos in tagged \
                 if (pos == 'NN' or pos == 'NNP' or pos == 'NNS' or pos == 'NNPS')]

        filter_keywords = ['chuck', 'norris', 'quot']
        filtered = [i for i in nouns if not any(f in i.lower() for f in filter_keywords)]
        return filtered

    def _make_absolute_path(self, path):
        if os.path.isabs(path):
            return path
        else:
            return str(os.path.join(os.getcwd(), path))

    @staticmethod
    def _filter_by_category(jokes, category):
        result = []
        for val in jokes:
            if category in val.get('categories'): result.append(val)
        return result

    @staticmethod
    def _filter_by_exclude_id(jokes, exclude_id_list):
        result = []
        for val in jokes:
            if val.get('id') not in exclude_id_list: result.append(val)
        return result


def main():
    PostcardsChuckNorris().main(sys.argv[1:])


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# encoding: utf-8

from postcards.postcards import Postcards
from postcards.plugin_folder.slice_image import make_tiles, store_tiles
import sys
import os
import random
import ntpath
from PIL import Image
import os
from time import gmtime, strftime


class PostcardsFolder(Postcards):
    """
    Send postcards with images from a local folder
    """

    supported_ext = ['.jpg', '.jpeg', '.png']

    def can_handle_command(self, command):
        return True if command in ['slice'] else False

    def handle_command(self, command, args):
        if command == 'slice':
            self.slice_image(source_image=self._make_absolute_path(args.picture),
                             tile_width=args.width, tile_height=args.height)

    def build_plugin_subparser(self, subparsers):
        parser_slice = subparsers.add_parser('slice', help='slice an image into tiles',
                                             description='slice an image into tiles to create a poster. \n'
                                                         + 'tiles need to be a multiple of 154x111 pixels '
                                                         + 'in order not to be cropped.')
        parser_slice.add_argument('picture',
                                  type=str,
                                  help='path to a picture to slice into tiles')

        parser_slice.add_argument('width',
                                  type=int,
                                  help='tile width')
        parser_slice.add_argument('height',
                                  type=int,
                                  help='tile height')

    def get_img_and_text(self, payload, cli_args):
        if not payload.get('folder'):
            self.logger.error("no folder set in configuration")
            exit(1)

        folder = self._make_absolute_path(payload.get('folder'))

        candidates = []
        for file in os.listdir(folder):
            for ext in self.supported_ext:
                if file.endswith(ext):
                    candidates.append(file)

        if not candidates:
            self.logger.error("no images in folder " + folder)
            exit(1)

        img_name = random.choice(candidates)
        img_path = os.path.join(folder, img_name)
        move_info = 'moving to sent folder' if payload.get('move') else 'no move'

        self.logger.info('choosing image {} ({})'.format(img_path, move_info))
        file = open(img_path, 'rb')

        if payload.get('move'):
            self._move_to_sent(folder, img_path)

        return {
            'img': file,
            'text': ''
        }

    def slice_image(self, source_image, tile_width, tile_height):
        if not os.path.isfile(source_image):
            self.logger.error('file {} does not exist'.format(source_image))
            exit(1)

        file = open(source_image, 'rb')
        with Image.open(file) as image:
            cwd = os.getcwd()
            basename = strftime("slice_%Y-%m-%d_%H-%M-%S", gmtime())
            directory = os.path.join(cwd, basename)

            self.logger.info('slicing picture {} into tiles'.format(source_image))
            tiles = make_tiles(image, tile_width=tile_width, tile_height=tile_height)
            store_tiles(tiles, directory)
            self.logger.info('picture sliced into {} tiles {}'.format(len(tiles), directory))

    def _move_to_sent(self, picture_folder, image_path):
        sent_folder = os.path.join(picture_folder, 'sent')
        if not os.path.exists(sent_folder):
            os.makedirs(sent_folder)
            self.logger.debug('creating folder {}'.format(sent_folder))

        img_name = self._get_filename(image_path)
        sent_img_path = os.path.join(sent_folder, img_name)
        os.rename(image_path, sent_img_path)
        self.logger.debug('moving image from {} to {}'.format(image_path, sent_img_path))

    def _get_filename(self, path):
        head, tail = ntpath.split(path)
        return tail or ntpath.basename(head)

    def _make_absolute_path(self, path):
        if not os.path.isabs(path):
            return os.path.join(os.getcwd(), path)
        else:
            return path


def main():
    PostcardsFolder().main(sys.argv[1:])


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# encoding: utf-8

from postcards.postcards import Postcards
from postcards.plugin_pexels.util.pexels import get_random_image_url, read_from_url
import sys


class PostcardsPexel(Postcards):
    """
    Send postcards with random images from pexels.com
    """

    def get_img_and_text(self, plugin_config, cli_args):
        url = get_random_image_url()
        self.logger.info('using pexels picture: ' + url)
        return {
            'img': read_from_url(url),
            'text': ''
        }


def main():
    PostcardsPexel().main(sys.argv[1:])


if __name__ == '__main__':
    main()



import random
import requests
from bs4 import BeautifulSoup
import os
import urllib
import sys

words_location = os.path.dirname(os.path.realpath(__file__)) + '/words.txt'
pexels_search_url = 'https://www.pexels.com/search/'

headers = {
    'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
    'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3',
    'Accept-Encoding': 'none',
    'Accept-Language': 'en-US,en;q=0.8',
    'Connection': 'keep-alive'}


def get_random_image(keyword=None):
    url = get_random_image_url(keyword=keyword)
    return read_from_url(url)


def read_from_url(url):
    request = urllib.request.Request(url, None, headers)  # The assembled request
    return urllib.request.urlopen(request)


def get_random_image_url(keyword=None, number=1, _count=0):
    words = read_words()
    if keyword:
        search_term = keyword
    else:
        search_term = random.choice(words)
    r = requests.get(pexels_search_url + search_term)
    soup = BeautifulSoup(r.text, 'html.parser')
    imgs = []

    for article in soup.findAll("article", {"class": "photo-item"}):
        src = article.a.img['src'].split('?')[0]
        imgs.append(src)

    if imgs:
        chosen = []
        if number > len(imgs):
            number = len(imgs)

        for i in range(0, number):
            img = random.choice(imgs)
            imgs.remove(img)
            chosen.append(img)

        if number is 1:
            return chosen[0]
        else:
            return chosen
    elif _count < 10 and not keyword:
        return get_random_image_url(_count=_count + 1)
    elif keyword:
        raise Exception("No image found for keyword: " + keyword)
    else:
        raise Exception(f"Something is broke, tried {count} times but no images")


def read_words():
    with open(words_location) as f:
        content = f.readlines()
        return [x.strip() for x in content]


if __name__ == '__main__':
    keyword = None
    number = 1
    if len(sys.argv) > 1:
        keyword = sys.argv[1]
    if len(sys.argv) > 2:
        number = int(sys.argv[2])
    try:
        imgs = get_random_image_url(keyword=keyword, number=number)
        if isinstance(imgs, str):
            print(imgs)
        else:
            for url in imgs:
                print(url)
    except Exception as e:
        print(e)
        # raise e



from setuptools import setup, find_packages
from pip.req import parse_requirements
import codecs
import os
import re

here = os.path.abspath(os.path.dirname(__file__))
install_reqs = parse_requirements(here + '/requirements.txt', session='hack')
reqs = [str(ir.req) for ir in install_reqs]

def read(*parts):
    return codecs.open(os.path.join(here, *parts), 'r').read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")

setup(
    name='postcards',
    version='0.0.8',
    url='http://github.com/abertschi/postcards',
    author='Andrin Bertschi',
    description='A CLI for the Swiss Postcard Creator',
    packages=find_packages(exclude=("tmp",)),
    platforms='any',
    keywords='postcard swiss',
    classifiers=[
    ],
    package_data={'postcards': ['template_config.json', 'plugin_pexels/util/words.txt',
                                'plugin_chuck_norris/chuck_norris_jokes.json']},
    setup_requires=[
    ],

    install_requires=reqs,
    entry_points={
        'console_scripts': ['postcards=postcards.postcards:main',
                            'postcards-folder=postcards.plugin_folder.postcards_folder:main',
                            'postcards-pexels=postcards.plugin_pexels.postcards_pexels:main',
                            'postcards-random=postcards.plugin_random.postcards_random:main',
                            'postcards-chuck-norris=postcards.plugin_chuck_norris.postcards_chuck_norris:main'],
    }
)



from PIL import Image
import os
from time import gmtime, strftime
import math
import sys
import logging

"""slice_image.py: Slice image into tiles."""

__author__ = "Andrin Bertschi. www.abertschi.ch"

LOGGER_NAME = 'slice_image'
logger = logging.getLogger(LOGGER_NAME)


def make_tiles(image, tile_width, tile_height):
    """
    slice PIL image to tiles

    :param image: PIL image
    :param tile_width: target tile width
    :param tile_height: target tile height
    :return: 2d array of PIL images
    """
    width_segments = math.floor(image.width / tile_width)
    height_segments = math.floor(image.height / tile_height)

    matrix = [[0 for i in range(width_segments)] for i in range(height_segments)]

    for h in range(height_segments):
        y_from = h * tile_height
        y_to = y_from + tile_height

        for w in range(width_segments):
            x_from = w * tile_width
            x_to = x_from + tile_width

            frame = image.crop((x_from, y_from, x_to, y_to))
            matrix[h][w] = frame
    return matrix


def store_tiles(tiles, directory, basename=None):
    """
    Store generated tiles to disk

    :param tiles: a 2d array of PIL images, as created by #make_tiles function
    :param directory: directory to store images
    :param basename: basename of image, if none is set, default name is chosen
    :return: nothing
    """
    if not basename:
        basename = strftime("sliced_%Y-%m-%d_%H-%M-%S", gmtime())

    if not os.path.exists(directory):
        os.makedirs(directory)
        logger.debug('creating {}'.format(directory))

    height = len(tiles)
    width = len(tiles[0])
    for h in range(height):
        for w in range(width):
            frame = tiles[h][w]
            filename = basename + '_{}-{}.jpg'.format(h, w)
            filepath = os.path.join(directory, filename)
            logger.debug('storing {}'.format(filepath))
            frame.save(filepath)


def _make_absolute_path(path):
    if os.path.isabs(path):
        return path
    else:
        return str(os.path.join(os.getcwd(), path))


def _from_cli(image_path, tile_width, tile_height):
    if not os.path.isfile(image_path):
        logger.error('file {} does not exist'.format(image_path))
        exit(1)

    file = open(image_path, 'rb')
    with Image.open(file) as image:
        cwd = os.getcwd()
        basename = strftime("sliced_%Y-%m-%d_%H-%M-%S", gmtime())
        directory = os.path.join(cwd, basename)

        tiles = make_tiles(image, tile_width=tile_width, tile_height=tile_height)
        store_tiles(tiles, directory)


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO,
                        format='%(name)s (%(levelname)s): %(message)s')
    logging.getLogger(LOGGER_NAME).setLevel(logging.DEBUG)

    if len(sys.argv) < 4:
        logger.error('wrong usage. call script python {} <image_path> <tile_width> <tile_height>'.format(sys.argv[0]))
        exit(1)

    image_path = _make_absolute_path(sys.argv[1])
    tile_height = int(sys.argv[3])
    tile_width = int(sys.argv[2])
    _from_cli(image_path, tile_width, tile_height)
